@@ -1,10 +1,10 @@
-# Copyright (c) 2008 Jonathan M. Lange. See LICENSE for details.
+# Copyright (c) 2008, 2009 Jonathan M. Lange. See LICENSE for details.
 
 """Test case related stuff."""
 
 __metaclass__ = type
 __all__ = [
-    'change_test_id',
+    'clone_test_with_new_id',
     'TestCase',
     'skip',
     'skipIf',
@@ -18,12 +18,13 @@ except ImportError:
     wraps = None
 import itertools
 import sys
+import types
 import unittest
 
 from testtools import content
+from testtools.compat import advance_iterator
 from testtools.runtest import RunTest
-from testtools.testresult import ExtendedToOriginalDecorator, TestResult
-from testtools.utils import advance_iterator
+from testtools.testresult import TestResult
 
 
 try:
@@ -56,7 +57,7 @@ except ImportError:
 
 class TestCase(unittest.TestCase):
     """Extensions to the basic TestCase.
-    
+
     :ivar exception_handlers: Exceptions to catch from setUp, runTest and
         tearDown. This list is able to be modified at any time and consists of
         (exception_class, handler(case, result, exception_value)) pairs.
@@ -75,6 +76,7 @@ class TestCase(unittest.TestCase):
         unittest.TestCase.__init__(self, *args, **kwargs)
         self._cleanups = []
         self._unique_id_gen = itertools.count(1)
+        self._traceback_id_gen = itertools.count(0)
         self.__setup_called = False
         self.__teardown_called = False
         self.__details = {}
@@ -119,7 +121,7 @@ class TestCase(unittest.TestCase):
     def shortDescription(self):
         return self.id()
 
-    def skip(self, reason):
+    def skipTest(self, reason):
         """Cause this test to be skipped.
 
         This raises self.skipException(reason). skipException is raised
@@ -132,6 +134,10 @@ class TestCase(unittest.TestCase):
         """
         raise self.skipException(reason)
 
+    # skipTest is how python2.7 spells this. Sometime in the future 
+    # This should be given a deprecation decorator - RBC 20100611.
+    skip = skipTest
+
     def _formatTypes(self, classOrIterable):
         """Format a class or a bunch of classes for display in an error."""
         className = getattr(classOrIterable, '__name__', None)
@@ -144,9 +150,10 @@ class TestCase(unittest.TestCase):
 
         See the docstring for addCleanup for more information.
 
-        Returns True if all cleanups ran without error, False otherwise.
+        :return: None if all cleanups ran without error, the most recently
+            raised exception from the cleanups otherwise.
         """
-        ok = True
+        last_exception = None
         while self._cleanups:
             function, arguments, keywordArguments = self._cleanups.pop()
             try:
@@ -154,15 +161,16 @@ class TestCase(unittest.TestCase):
             except KeyboardInterrupt:
                 raise
             except:
-                self._report_error(self, result, None)
-                ok = False
-        return ok
+                exc_info = sys.exc_info()
+                self._report_traceback(exc_info)
+                last_exception = exc_info[1]
+        return last_exception
 
     def addCleanup(self, function, *arguments, **keywordArguments):
-        """Add a cleanup function to be called before tearDown.
+        """Add a cleanup function to be called after tearDown.
 
         Functions added with addCleanup will be called in reverse order of
-        adding after the test method and before tearDown.
+        adding after tearDown, or after setUp if setUp raises an exception.
 
         If a function added with addCleanup raises an exception, the error
         will be recorded as a test error, and the next cleanup will then be
@@ -182,7 +190,7 @@ class TestCase(unittest.TestCase):
         which is expensive to calculate and not interesting for reporting in
         the success case.
 
-        Handlers are called before the outcome (such as addFailure) that 
+        Handlers are called before the outcome (such as addFailure) that
         the exception has caused.
 
         Handlers are called in first-added, first-called order, and if they
@@ -195,22 +203,33 @@ class TestCase(unittest.TestCase):
     def _add_reason(self, reason):
         self.addDetail('reason', content.Content(
             content.ContentType('text', 'plain'),
-            lambda:[reason.encode('utf8')]))
+            lambda: [reason.encode('utf8')]))
 
     def assertIn(self, needle, haystack):
         """Assert that needle is in haystack."""
         self.assertTrue(
             needle in haystack, '%r not in %r' % (needle, haystack))
 
-    def assertIs(self, expected, observed):
-        """Assert that `expected` is `observed`."""
-        self.assertTrue(
-            expected is observed, '%r is not %r' % (expected, observed))
+    def assertIs(self, expected, observed, message=''):
+        """Assert that 'expected' is 'observed'.
 
-    def assertIsNot(self, expected, observed):
-        """Assert that `expected` is not `observed`."""
+        :param expected: The expected value.
+        :param observed: The observed value.
+        :param message: An optional message describing the error.
+        """
+        if message:
+            message = ': ' + message
         self.assertTrue(
-            expected is not observed, '%r is %r' % (expected, observed))
+            expected is observed,
+            '%r is not %r%s' % (expected, observed, message))
+
+    def assertIsNot(self, expected, observed, message=''):
+        """Assert that 'expected' is not 'observed'."""
+        if message:
+            message = ': ' + message
+        self.assertTrue(
+            expected is not observed,
+            '%r is %r%s' % (expected, observed, message))
 
     def assertNotIn(self, needle, haystack):
         """Assert that needle is not in haystack."""
@@ -249,6 +268,14 @@ class TestCase(unittest.TestCase):
         mismatch = matcher.match(matchee)
         if not mismatch:
             return
+        existing_details = self.getDetails()
+        for (name, content) in mismatch.get_details().items():
+            full_name = name
+            suffix = 1
+            while full_name in existing_details:
+                full_name = "%s-%d" % (name, suffix)
+                suffix += 1
+            self.addDetail(full_name, content)
         self.fail('Match failed. Matchee: "%s"\nMatcher: %s\nDifference: %s\n'
             % (matchee, matcher, mismatch.describe()))
 
@@ -276,29 +303,48 @@ class TestCase(unittest.TestCase):
             predicate(*args, **kwargs)
         except self.failureException:
             exc_info = sys.exc_info()
-            self.addDetail('traceback',
-                content.TracebackContent(exc_info, self))
+            self._report_traceback(exc_info)
             raise _ExpectedFailure(exc_info)
         else:
             raise _UnexpectedSuccess(reason)
 
     def getUniqueInteger(self):
+        """Get an integer unique to this test.
+
+        Returns an integer that is guaranteed to be unique to this instance.
+        Use this when you need an arbitrary integer in your test, or as a
+        helper for custom anonymous factory methods.
+        """
         return advance_iterator(self._unique_id_gen)
 
-    def getUniqueString(self):
-        return '%s-%d' % (self.id(), self.getUniqueInteger())
+    def getUniqueString(self, prefix=None):
+        """Get a string unique to this test.
+
+        Returns a string that is guaranteed to be unique to this instance. Use
+        this when you need an arbitrary string in your test, or as a helper
+        for custom anonymous factory methods.
+
+        :param prefix: The prefix of the string. If not provided, defaults
+            to the id of the tests.
+        :return: A bytestring of '<prefix>-<unique_int>'.
+        """
+        if prefix is None:
+            prefix = self.id()
+        return '%s-%d' % (prefix, self.getUniqueInteger())
 
     def onException(self, exc_info):
         """Called when an exception propogates from test code.
 
         :seealso addOnException:
         """
+        if exc_info[0] not in [
+            TestSkipped, _UnexpectedSuccess, _ExpectedFailure]:
+            self._report_traceback(exc_info)
         for handler in self.__exception_handlers:
             handler(exc_info)
 
     @staticmethod
     def _report_error(self, result, err):
-        self._report_traceback()
         result.addError(self, details=self.getDetails())
 
     @staticmethod
@@ -307,7 +353,6 @@ class TestCase(unittest.TestCase):
 
     @staticmethod
     def _report_failure(self, result, err):
-        self._report_traceback()
         result.addFailure(self, details=self.getDetails())
 
     @staticmethod
@@ -319,9 +364,13 @@ class TestCase(unittest.TestCase):
         self._add_reason(reason)
         result.addSkip(self, details=self.getDetails())
 
-    def _report_traceback(self):
-        self.addDetail('traceback',
-            content.TracebackContent(sys.exc_info(), self))
+    def _report_traceback(self, exc_info):
+        tb_id = advance_iterator(self._traceback_id_gen)
+        if tb_id:
+            tb_label = 'traceback-%d' % tb_id
+        else:
+            tb_label = 'traceback'
+        self.addDetail(tb_label, content.TracebackContent(exc_info, self))
 
     @staticmethod
     def _report_unexpected_success(self, result, err):
@@ -339,8 +388,12 @@ class TestCase(unittest.TestCase):
         """
         self.setUp()
         if not self.__setup_called:
-            raise ValueError("setUp was not called")
-    
+            raise ValueError(
+                "TestCase.setUp was not called. Have you upcalled all the "
+                "way up the hierarchy from your setUp? e.g. Call "
+                "super(%s, self).setUp() from your setUp()."
+                % self.__class__.__name__)
+
     def _run_teardown(self, result):
         """Run the tearDown function for this test.
 
@@ -350,7 +403,11 @@ class TestCase(unittest.TestCase):
         """
         self.tearDown()
         if not self.__teardown_called:
-            raise ValueError("teardown was not called")
+            raise ValueError(
+                "TestCase.tearDown was not called. Have you upcalled all the "
+                "way up the hierarchy from your tearDown? e.g. Call "
+                "super(%s, self).tearDown() from your tearDown()."
+                % self.__class__.__name__)
 
     def _run_test_method(self, result):
         """Run the test method for this test.
@@ -376,9 +433,19 @@ class TestCase(unittest.TestCase):
         self.__teardown_called = True
 
 
+# Python 2.4 did not know how to copy functions.
+if types.FunctionType not in copy._copy_dispatch:
+    copy._copy_dispatch[types.FunctionType] = copy._copy_immutable
+
+
+
 def clone_test_with_new_id(test, new_id):
-    """Copy a TestCase, and give the copied test a new id."""
-    newTest = copy.deepcopy(test)
+    """Copy a TestCase, and give the copied test a new id.
+    
+    This is only expected to be used on tests that have been constructed but
+    not executed.
+    """
+    newTest = copy.copy(test)
     newTest.id = lambda: new_id
     return newTest
 
@@ -418,4 +485,3 @@ def skipUnless(condition, reason):
     def _id(obj):
         return obj
     return _id
-
