@@ -13,6 +13,8 @@ __all__ = [
 import datetime
 import unittest
 
+from testtools.compat import _format_exc_info, str_is_unicode, _u
+
 
 class TestResult(unittest.TestResult):
     """Subclass of unittest.TestResult extending the protocol for flexability.
@@ -105,15 +107,32 @@ class TestResult(unittest.TestResult):
         """Called when a test was expected to fail, but succeed."""
         self.unexpectedSuccesses.append(test)
 
+    if str_is_unicode:
+        # Python 3 and IronPython strings are unicode, use parent class method
+        _exc_info_to_unicode = unittest.TestResult._exc_info_to_string
+    else:
+        # For Python 2, need to decode components of traceback according to
+        # their source, so can't use traceback.format_exception
+        # Here follows a little deep magic to copy the existing method and
+        # replace the formatter with one that returns unicode instead
+        from types import FunctionType as __F, ModuleType as __M
+        __f = unittest.TestResult._exc_info_to_string.im_func
+        __g = dict(__f.func_globals)
+        __m = __M("__fake_traceback")
+        __m.format_exception = _format_exc_info
+        __g["traceback"] = __m
+        _exc_info_to_unicode = __F(__f.func_code, __g, "_exc_info_to_unicode")
+        del __F, __M, __f, __g, __m
+
     def _err_details_to_string(self, test, err=None, details=None):
         """Convert an error in exc_info form or a contents dict to a string."""
         if err is not None:
-            return self._exc_info_to_string(err, test)
+            return self._exc_info_to_unicode(err, test)
         return _details_to_str(details)
 
     def _now(self):
         """Return the current 'test time'.
-        
+
         If the time() method has not been called, this is equivalent to
         datetime.now(), otherwise its the last supplied datestamp given to the
         time() method.
@@ -142,7 +161,7 @@ class TestResult(unittest.TestResult):
         concurrently and getting the system time between API calls will not
         accurately represent the duration of tests (or the whole run).
 
-        Calling time() sets the datetime used by the TestResult object. 
+        Calling time() sets the datetime used by the TestResult object.
         Time is permitted to go backwards when using this call.
 
         :param a_datetime: A datetime.datetime object with TZ information or
@@ -152,7 +171,7 @@ class TestResult(unittest.TestResult):
 
     def done(self):
         """Called when the test runner is done.
-        
+
         deprecated in favour of stopTestRun.
         """
 
@@ -237,7 +256,8 @@ class TextTestResult(TestResult):
         self._show_list('ERROR', self.errors)
         self._show_list('FAIL', self.failures)
         self.stream.write("Ran %d test%s in %.3fs\n\n" %
-            (self.testsRun, plural, self._delta_to_float(stop-self.__start)))
+            (self.testsRun, plural,
+             self._delta_to_float(stop - self.__start)))
         if self.wasSuccessful():
             self.stream.write("OK\n")
         else:
@@ -252,7 +272,7 @@ class TextTestResult(TestResult):
 
 class ThreadsafeForwardingResult(TestResult):
     """A TestResult which ensures the target does not receive mixed up calls.
-    
+
     This is used when receiving test results from multiple sources, and batches
     up all the activity for a single test into a thread-safe batch where all
     other ThreadsafeForwardingResult objects sharing the same semaphore will be
@@ -382,7 +402,8 @@ class ExtendedToOriginalDecorator(object):
 
     def addExpectedFailure(self, test, err=None, details=None):
         self._check_args(err, details)
-        addExpectedFailure = getattr(self.decorated, 'addExpectedFailure', None)
+        addExpectedFailure = getattr(
+            self.decorated, 'addExpectedFailure', None)
         if addExpectedFailure is None:
             return self.addSuccess(test)
         if details is not None:
@@ -506,13 +527,23 @@ class ExtendedToOriginalDecorator(object):
 class _StringException(Exception):
     """An exception made from an arbitrary string."""
 
+    if not str_is_unicode:
+        def __init__(self, string):
+            if type(string) is not unicode:
+                raise TypeError("_StringException expects unicode, got %r" %
+                    (string,))
+            Exception.__init__(self, string)
+
+        def __str__(self):
+            return self.args[0].encode("utf-8")
+
+        def __unicode__(self):
+            return self.args[0]
+    # For 3.0 and above the default __str__ is fine, so we don't define one.
+
     def __hash__(self):
         return id(self)
 
-    def __str__(self):
-        """Stringify better than 2.x's default behaviour of ascii encoding."""
-        return self.args[0]
-
     def __eq__(self, other):
         try:
             return self.args == other.args
@@ -535,5 +566,4 @@ def _details_to_str(details):
         if not chars[-1].endswith('\n'):
             chars.append('\n')
         chars.append('------------\n')
-    return ''.join(chars)
-
+    return _u(''.join(chars))
