@@ -1,7 +1,10 @@
+"""Tests for testtools itself."""
+
 # See README for copyright and licensing details.
 
 import unittest
 from testtools.tests import (
+    test_compat,
     test_content,
     test_content_type,
     test_matchers,
@@ -15,6 +18,7 @@ from testtools.tests import (
 def test_suite():
     suites = []
     modules = [
+        test_compat,
         test_content,
         test_content_type,
         test_matchers,
